use bevy::prelude::*;

use crate::components::{Bullet, Enemy, EnemyBullet, EnemyType};
use crate::constants::{
    BULLET_ENEMY_COLLISION_THRESHOLD, BULLET_SIZE, BULLET_SPEED, ENEMY_BULLET_PLAYER_COLLISION_THRESHOLD,
    ENEMY_BULLET_SIZE, ENEMY_BULLET_SPEED, WINDOW_HEIGHT,
};
use crate::resources::{PlayerLives, PlayerRespawnTimer, Score};
use crate::game_state::AppState;
use crate::components::Player; // Needed for check_enemy_bullet_player_collisions
use crate::components::Invincible; // Needed for check_enemy_bullet_player_collisions

// --- Player Bullet Systems ---

pub fn move_bullets(
    mut query: Query<(Entity, &mut Transform), With<Bullet>>,
    time: Res<Time>,
    mut commands: Commands,
) {
    for (entity, mut transform) in query.iter_mut() {
        transform.translation.y += BULLET_SPEED * time.delta_seconds();

        if transform.translation.y > WINDOW_HEIGHT / 2.0 + BULLET_SIZE.y {
            commands.entity(entity).despawn();
        }
    }
}

pub fn check_bullet_collisions(
    mut commands: Commands,
    bullet_query: Query<(Entity, &Transform), With<Bullet>>,
    enemy_query: Query<(Entity, &Transform, &Enemy), With<Enemy>>, // Fetch Enemy component too
    mut score: ResMut<Score>,                                      // Add Score resource
) {
    for (bullet_entity, bullet_transform) in bullet_query.iter() {
        for (enemy_entity, enemy_transform, enemy) in enemy_query.iter() {
            // Get Enemy component
            let distance = bullet_transform
                .translation
                .distance(enemy_transform.translation);

            if distance < BULLET_ENEMY_COLLISION_THRESHOLD {
                commands.entity(bullet_entity).despawn();
                commands.entity(enemy_entity).despawn();
                // Increment score based on enemy type
                let points = match enemy.enemy_type {
                    EnemyType::Grunt => 100,
                    EnemyType::Boss => 100, // Same points as Grunt for now
                };
                score.value += points;
                println!("Enemy hit by player bullet! Score: {}", score.value); // Log score update
                break; // Bullet only hits one enemy
            }
        }
    }
}

// --- Enemy Bullet Systems ---

pub fn move_enemy_bullets(
    mut commands: Commands,
    time: Res<Time>,
    mut query: Query<(Entity, &mut Transform), With<EnemyBullet>>,
) {
    for (entity, mut transform) in query.iter_mut() {
        transform.translation.y -= ENEMY_BULLET_SPEED * time.delta_seconds();

        // Despawn if off screen (bottom)
        if transform.translation.y < -WINDOW_HEIGHT / 2.0 - ENEMY_BULLET_SIZE.y {
            commands.entity(entity).despawn();
        }
    }
}

// Check collisions between enemy bullets and the player
pub fn check_enemy_bullet_player_collisions(
    mut commands: Commands,
    mut lives: ResMut<PlayerLives>,
    mut respawn_timer: ResMut<PlayerRespawnTimer>,
    mut next_state: ResMut<NextState<AppState>>,
    // Player query matching the run_if condition (player exists and is vulnerable)
    player_query: Query<(Entity, &Transform), (With<Player>, Without<Invincible>)>,
    enemy_bullet_query: Query<(Entity, &Transform), With<EnemyBullet>>,
) {
    if let Ok((player_entity, player_transform)) = player_query.get_single() {
        for (bullet_entity, bullet_transform) in enemy_bullet_query.iter() {
            let distance = player_transform
                .translation
                .distance(bullet_transform.translation);

            if distance < ENEMY_BULLET_PLAYER_COLLISION_THRESHOLD {
                println!("Player hit by enemy bullet!");
                commands.entity(bullet_entity).despawn(); // Despawn bullet

                lives.count = lives.count.saturating_sub(1);
                println!("Lives remaining: {}", lives.count);

                commands.entity(player_entity).despawn(); // Despawn player

                if lives.count > 0 {
                    respawn_timer.timer.reset();
                    respawn_timer.timer.unpause();
                    println!("Respawn timer started.");
                } else {
                    println!("GAME OVER!");
                    next_state.set(AppState::GameOver);
                }
                // Break because the player can only be hit once per frame
                break;
            }
        }
    }
}