use bevy::prelude::*;
use std::time::Duration;

use crate::components::{Enemy, EnemyBullet, EnemyState, EnemyType, FormationTarget};
use crate::constants::{
    ENEMY_BULLET_PLAYER_COLLISION_THRESHOLD, ENEMY_BULLET_SIZE, ENEMY_BULLET_SPEED,
    ENEMY_SHOOT_INTERVAL, ENEMY_SIZE, ENEMY_SPEED, FORMATION_BASE_Y, FORMATION_COLS,
    FORMATION_ENEMY_COUNT, FORMATION_X_SPACING, FORMATION_Y_SPACING, WINDOW_HEIGHT,
};
use crate::resources::{
    AttackDiveTimer, CurrentStage, EnemySpawnTimer, FormationState, PlayerLives,
    PlayerRespawnTimer,
};
use crate::game_state::AppState; // Needed for check_enemy_bullet_player_collisions

pub fn spawn_enemies(
    mut commands: Commands,
    time: Res<Time>,
    mut timer: ResMut<EnemySpawnTimer>,
    mut stage: ResMut<CurrentStage>,
    mut formation_state: ResMut<FormationState>, // Add formation state resource
) {
    // Tick the timer every frame
    timer.timer.tick(time.delta());

    // Only spawn if we haven't spawned the full formation for this stage yet
    // AND the timer just finished this frame
    if formation_state.total_spawned_this_stage < FORMATION_ENEMY_COUNT
        && timer.timer.just_finished()
    {
        // Calculate grid position
        let slot_index = formation_state.next_slot_index;
        let row = slot_index / FORMATION_COLS;
        let col = slot_index % FORMATION_COLS;

        // Calculate target position in formation
        let target_x = (col as f32 - (FORMATION_COLS as f32 - 1.0) / 2.0) * FORMATION_X_SPACING;
        let target_y = FORMATION_BASE_Y - (row as f32 * FORMATION_Y_SPACING);
        let target_pos = Vec3::new(target_x, target_y, 0.0);

        // Spawn position (still random at the top for now)
        let spawn_x = (fastrand::f32() - 0.5) * (WINDOW_HEIGHT - ENEMY_SIZE.x); // Use WINDOW_HEIGHT for x spawn range? Should be WINDOW_WIDTH
        let spawn_y = WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y / 2.0;

        // Determine enemy type (currently always Grunt)
        let enemy_type = EnemyType::Grunt;

        // Determine sprite color based on type
        let sprite_color = match enemy_type {
            EnemyType::Grunt => Color::rgb(1.0, 0.2, 0.2), // Red for Grunts
            EnemyType::Boss => Color::rgb(0.8, 0.2, 1.0),  // Purple for Bosses
        };

        commands.spawn((
            SpriteBundle {
                sprite: Sprite {
                    color: sprite_color, // Use determined color
                    custom_size: Some(ENEMY_SIZE),
                    ..default()
                },
                transform: Transform::from_translation(Vec3::new(spawn_x, spawn_y, 0.0)),
                ..default()
            },
            Enemy {
                enemy_type, // Use the defined type
                // Initialize cooldown, maybe slightly randomized later
                shoot_cooldown: Timer::new(
                    Duration::from_secs_f32(ENEMY_SHOOT_INTERVAL),
                    TimerMode::Once,
                ),
            },
            FormationTarget {
                position: target_pos,
            },
            EnemyState::Entering, // Start in Entering state
        ));

        formation_state.next_slot_index = (slot_index + 1) % FORMATION_ENEMY_COUNT; // Cycle through slots if needed, though we stop spawning
        formation_state.total_spawned_this_stage += 1;

        // Mark that we are now waiting for enemies to be cleared
        stage.waiting_for_clear = true;

        // Reset timer only if we are still spawning more enemies for the formation
        if formation_state.total_spawned_this_stage < FORMATION_ENEMY_COUNT {
            timer.timer.reset();
        } else {
            println!("Full formation spawned for Stage {}", stage.number);
        }
    }
}

pub fn move_enemies(
    mut entering_query: Query<
        (Entity, &mut Transform, &FormationTarget, &mut EnemyState),
        (With<Enemy>, With<FormationTarget>),
    >,
    mut attacking_query: Query<
        (Entity, &mut Transform, &EnemyState, &Enemy), // Add &Enemy here
        (With<Enemy>, Without<FormationTarget>),
    >, // Query potential attackers
    time: Res<Time>,
    mut commands: Commands,
    stage: Res<CurrentStage>, // Add stage resource for speed calculation
) {
    // Increase speed based on stage number
    let current_speed = ENEMY_SPEED + (stage.number - 1) as f32 * 10.0;
    let arrival_threshold = current_speed * time.delta_seconds() * 1.1; // Slightly more than one frame's movement

    // --- Handle Entering Enemies ---
    for (entity, mut transform, target, mut state) in entering_query.iter_mut() {
        // Ensure we only process Entering state here, though query filters it mostly
        if *state == EnemyState::Entering {
            let current_pos = transform.translation;
            let target_pos = target.position;
            let direction = target_pos - current_pos;
            let distance = direction.length();

            if distance < arrival_threshold {
                // Arrived at target
                transform.translation = target_pos;
                commands.entity(entity).remove::<FormationTarget>(); // Remove target component
                *state = EnemyState::InFormation; // Change state
                println!(
                    "Enemy {:?} reached formation target and is now InFormation.",
                    entity
                );
            } else {
                // Move towards target
                let move_delta = direction.normalize() * current_speed * time.delta_seconds();
                transform.translation += move_delta;
            }
        }
    }

    // --- Handle Attacking Enemies ---
    let attack_speed = current_speed * 1.5; // Make them dive faster
    for (entity, mut transform, state, enemy) in attacking_query.iter_mut() {
        // Get state and enemy
        // *** Explicitly check if the enemy is actually in the Attacking state ***
        if *state == EnemyState::Attacking {
            // --- Apply movement based on EnemyType ---
            match enemy.enemy_type {
                EnemyType::Grunt | EnemyType::Boss => { // Combine logic if it's the same
                    // Swooping Dive Logic
                    let delta_seconds = time.delta_seconds();
                    let vertical_movement = attack_speed * delta_seconds;

                    // Horizontal movement: Move towards the center (x=0)
                    let horizontal_speed_factor = 0.5; // Adjust this to control the swoop intensity
                    let horizontal_movement = if transform.translation.x < 0.0 {
                        attack_speed * horizontal_speed_factor * delta_seconds
                    } else if transform.translation.x > 0.0 {
                        -attack_speed * horizontal_speed_factor * delta_seconds
                    } else {
                        0.0 // No horizontal movement if exactly at center
                    };

                    // Apply movement
                    transform.translation.y -= vertical_movement;
                    transform.translation.x += horizontal_movement;

                    // Ensure enemy doesn't overshoot the center horizontally if moving towards it
                    if (transform.translation.x - horizontal_movement < 0.0
                        && transform.translation.x > 0.0)
                        || (transform.translation.x - horizontal_movement > 0.0
                            && transform.translation.x < 0.0)
                    {
                        transform.translation.x = 0.0;
                    }
                } // Add cases for other enemy types later if different
            } // Closes match enemy.enemy_type
        } // Closes if *state == EnemyState::Attacking

        // Despawn if off screen (This should be inside the loop)
        if transform.translation.y < -WINDOW_HEIGHT / 2.0 - ENEMY_SIZE.y {
            println!(
                "Despawning enemy {:?} that went off screen.", // Generic message as it could be InFormation or Attacking
                entity
            );
            commands.entity(entity).despawn();
            // TODO: Later, attacking enemies might return to formation or loop
        }
    } // Closes for loop
}

// System to check if all spawned enemies have reached their formation position
pub fn check_formation_complete(
    mut formation_state: ResMut<FormationState>,
    enemy_query: Query<&EnemyState, With<Enemy>>, // Query states directly
    mut attack_dive_timer: ResMut<AttackDiveTimer>, // Add timer to reset it
) {
    // Only run the check if the formation isn't already marked as complete
    if !formation_state.formation_complete {
        // Check if all enemies for the stage have been spawned
        if formation_state.total_spawned_this_stage == FORMATION_ENEMY_COUNT {
            // Check if any enemies are still in the Entering state
            let mut any_entering = false;
            for state in enemy_query.iter() {
                if *state == EnemyState::Entering {
                    any_entering = true;
                    break;
                }
            }

            // If all spawned and none are entering, formation is complete
            if !any_entering {
                println!(
                    "Formation complete! Resetting attack timer. (Spawned={})",
                    formation_state.total_spawned_this_stage
                );
                formation_state.formation_complete = true;
                attack_dive_timer.timer.reset(); // Reset the dive timer
                attack_dive_timer.timer.unpause(); // Start the timer now
                println!("Formation complete! Attack timer unpaused and reset.");
            }
        }
    }
}

pub fn trigger_attack_dives(
    mut timer: ResMut<AttackDiveTimer>,
    time: Res<Time>,
    mut query: Query<(Entity, &mut EnemyState), With<Enemy>>,
    formation_state: Res<FormationState>, // Read formation state
) {
    timer.timer.tick(time.delta());

    // Only proceed if the timer finished AND the formation is complete
    if timer.timer.just_finished() && formation_state.formation_complete {
        // Find all enemies currently in formation
        let mut available_enemies: Vec<Entity> = Vec::new();
        for (entity, state) in query.iter() {
            if *state == EnemyState::InFormation {
                available_enemies.push(entity);
            }
        }

        // If there are enemies available, pick one randomly
        if !available_enemies.is_empty() {
            let random_index = fastrand::usize(..available_enemies.len());
            let chosen_entity = available_enemies[random_index];

            // Get the chosen enemy's state mutably and change it
            if let Ok((_, mut state)) = query.get_mut(chosen_entity) {
                println!("Enemy {:?} starting attack dive!", chosen_entity);
                *state = EnemyState::Attacking;
                // Timer will automatically repeat due to TimerMode::Repeating
            }
        }
    }
}

pub fn enemy_shoot(
    mut commands: Commands,
    time: Res<Time>,
    // Query attacking enemies: need their transform and mutable Enemy component for the timer
    mut enemy_query: Query<(&Transform, &mut Enemy, &EnemyState), Without<FormationTarget>>,
) {
    for (transform, mut enemy, state) in enemy_query.iter_mut() {
        // Only shoot if attacking
        if *state == EnemyState::Attacking {
            enemy.shoot_cooldown.tick(time.delta());
            if enemy.shoot_cooldown.finished() {
                println!("Enemy {:?} firing!", transform.translation); // Placeholder for entity ID if needed
                let bullet_start_pos = transform.translation
                    - Vec3::Y * (ENEMY_SIZE.y / 2.0 + ENEMY_BULLET_SIZE.y / 2.0);

                commands.spawn((
                    SpriteBundle {
                        sprite: Sprite {
                            color: Color::rgb(1.0, 0.5, 0.5), // Different color for enemy bullets
                            custom_size: Some(ENEMY_BULLET_SIZE),
                            ..default()
                        },
                        transform: Transform::from_translation(bullet_start_pos),
                        ..default()
                    },
                    EnemyBullet,
                ));

                // Reset the timer for the next shot
                enemy.shoot_cooldown.reset();
            }
        }
    }
}

// Check collisions between enemy bullets and the player
// Moved to player.rs as it affects player state directly

// New run condition: Check if the formation is complete
pub fn is_formation_complete(formation_state: Res<FormationState>) -> bool {
    formation_state.formation_complete
}